import unittest
import mock

import rvrbase
from rvrbase.constants import ERR_POST_DATA


class Testrvrconfig(unittest.TestCase):

    def test_rvrconfig_wrong_file(self):
        with self.assertRaises(FileNotFoundError):
            rvrbase.Rvrbase('filedoesnotexist.ini')

    @mock.patch('requests.post')
    def test_rvrconfig_q1_success(self, mock_requests_post):
        mock_requests_post.return_value = mock.Mock()
        mock_requests_post.return_value.status_code = 200

        config = rvrbase.Rvrbase('src/test/testdata/config.ini')
        self.assertEqual(config.q1('workspace_id'),
                         '7756814b-7720-4b6d-9fb6-0aa03fe97658')

    @mock.patch('requests.post')
    def test_rvrconfig_q1_non_existing_key(self, mock_requests_post):
        mock_requests_post.return_value = mock.Mock()
        mock_requests_post.return_value.status_code = 200

        config = rvrbase.Rvrbase('src/test/testdata/config.ini')
        self.assertEqual(config.q1('doesnotexist'), None)

    # Happy flow of all the functionality in delegate.
    @mock.patch('requests.post')
    @mock.patch('notify_run.Notify.send')
    @mock.patch('logging.info')
    @mock.patch('logging.warning')
    @mock.patch('logging.error')
    def test_log_application_event_success2(self, mock_logging_error, mock_logging_warning,
                                            mock_logging_info, mock_notify, mock_requests_post):
        mock_requests_post.return_value = mock.Mock()
        mock_requests_post.return_value.status_code = 200

        rvrlogger = rvrbase.Rvrbase('src/test/testdata/config.ini')
        rvrlogger.log_app_event(
            type='warning', message='No worries, just testing here.', notify_message=True)

        rvrlogger.log_app_event(
            type='error', message='No worries, just testing here.', notify_message=True)

        mock_requests_post.assert_called()
        mock_notify.assert_called()
        mock_logging_info.assert_called()  # should always be called with starting message
        mock_logging_warning.assert_called()
        mock_logging_error.assert_called()

    # Unhappy flow send_az_app_event
    @mock.patch('rvrbase.api.azure_logger_api')
    @mock.patch('logging.error')
    @mock.patch('notify_run.Notify.send')
    def test_send_az_app_event_error(self, mock_notify, mock_logging_error, mock_azure_logger_api):
        mock_azure_logger_api.side_effect = RuntimeError(
            mock.Mock(ERR_POST_DATA.format(status_code=404)))
        # barMock.side_effect = HttpError(mock.Mock(status=404), 'not found')

        rvrbase.Rvrbase('src/test/testdata/config.ini')

        # An error should be logged to logger and send using notify.
        mock_logging_error.assert_called()
        mock_notify.assert_called()


if __name__ == '__main__':
    unittest.main()



# from https://medium.com/slalom-build/reading-and-writing-to-azure-log-analytics-c78461056862
import requests
import hashlib
import hmac
import base64
import logging
import datetime
from rvrbase.constants import ERR_POST_DATA, MSG_POST_DATA_SUCCESS


class Azure_logger_api():
    def _build_signature(self, customer_id, shared_key, date, content_length, method, content_type,
                         resource):
        # Returns authorization header which will be used when sending data into Azure Log
        # Analytics

        x_headers = 'x-ms-date:' + date
        string_to_hash = method + "\n" + \
            str(content_length) + "\n" + content_type + \
            "\n" + x_headers + "\n" + resource
        bytes_to_hash = bytes(string_to_hash, 'UTF-8')
        decoded_key = base64.b64decode(shared_key)
        encoded_hash = base64.b64encode(hmac.new(
            decoded_key, bytes_to_hash, digestmod=hashlib.sha256).digest()).decode('utf-8')
        authorization = "SharedKey {}:{}".format(customer_id, encoded_hash)
        return authorization

    def post_data(self, body, log_type, workspace_id, workspace_prim_key):
        """Sends payload to Azure Log Analytics Workspace

        Keyword arguments:
        customer_id -- Workspace ID obtained from Advanced Settings
        shared_key -- Authorization header, created using build_signature
        body -- payload to send to Azure Log Analytics
        log_type -- Azure Log Analytics table name
        """

        method = 'POST'
        content_type = 'application/json'
        resource = '/api/logs'
        rfc1123date = datetime.datetime.utcnow().strftime('%a, %d %b %Y %H:%M:%S GMT')
        content_length = len(body)
        signature = self._build_signature(
            workspace_id, workspace_prim_key, rfc1123date, content_length, method, content_type,
            resource)

        uri = 'https://' + workspace_id + '.ods.opinsights.azure.com' + \
            resource + '?api-version=2016-04-01'

        headers = {
            'content-type': content_type,
            'Authorization': signature,
            'Log-Type': log_type,
            'x-ms-date': rfc1123date
        }

        response = requests.post(uri, data=body, headers=headers)
        if (response.status_code >= 200 and response.status_code <= 299):
            logging.info(MSG_POST_DATA_SUCCESS.format(body=body))
        else:
            raise RuntimeError(ERR_POST_DATA.format(
                status_code=response.status_code))



from notify_run import Notify


class New_notification_api:
    def __init__(self):
        super().__init__()
        self.notify = Notify()

    def send_browser_notification(self, message):
        self.notify.send(message)



from configparser import ConfigParser
from pathlib import Path
import logging
import errno
import os


class Rvrconfig:

    def __init__(self, path):
        self.path = path

        configfile = Path(path)

        if configfile.exists():
            logging.debug('configfile {} exists'.format(path))
            self.readconfig()
        else:
            raise FileNotFoundError(
                errno.ENOENT, os.strerror(errno.ENOENT), path)

    def readconfig(self):
        self.config = ConfigParser()
        self.config.read(self.path)

    # Very small implementation for querying configuration INI files.
    # It simply assumes all keys are unique and returns a key if found.
    def query_configfile(self, key):
        for section in self.config.sections():
            if key in self.config[section].keys():
                return self.config[section][key]

    # This function is to be extended with more query functionality. For example arguments.
    def q1(self, key):
        return self.query_configfile(key=key)



CONFIG_FILE = '/etc/rvr/config.ini'

LOG_TYPE_APPLICATION_EVENT = 'app_event'

VALID_MESSAGE_TYPES = {'debug', 'info', 'warning', 'error'}

NOTIFY_APPLICATION_EVENT = '{type} on {node}: {message}'

# Messages
MSG_POST_DATA_SUCCESS = 'Accepted payload: {body}'
MST_STARTING = 'Starting, notify URL:{url}'

# Errors:
ERR_POST_DATA = 'Unable to Write: {status_code}'
ERR_POST_DATA_TO_AZURE = 'Failed to post data to Azure. Error: {message}'



import logging
from platform import node
import json
import sys
from dbus import ValidationException

from rvrbase.__about__ import __version__ as version
from rvrbase.api import New_notification_api
from rvrbase.api import Azure_logger_api
from rvrbase.api import Rvrconfig
from rvrbase.constants import LOG_TYPE_APPLICATION_EVENT, MST_STARTING, NOTIFY_APPLICATION_EVENT,\
    VALID_MESSAGE_TYPES


class Rvrbase():

    def __init__(self, path):

        self.config_api = Rvrconfig(path)
        self.notifcation_api = New_notification_api()
        self.azure_logger_api = Azure_logger_api()
        self.workspace_id = self.q1('workspace_id')
        self.workspace_prim_key = self.q1('primary_key')

        self.send_az_app_event(type='info', message=MST_STARTING.format(
            url=self.notifcation_api.notify.endpoint))

    def send_browser_notification(self, message, type):
        if type not in VALID_MESSAGE_TYPES:
            raise ValidationException(
                f'{type} is not in {VALID_MESSAGE_TYPES}.')

        self.notifcation_api.send_browser_notification(
            NOTIFY_APPLICATION_EVENT.format(type=type, node=node(), message=message))

    def log_app_event(self, type, message, notify_message=False):
        if type not in VALID_MESSAGE_TYPES:
            raise ValidationException(
                f'{type} is not in {VALID_MESSAGE_TYPES}.')

        if type == 'debug':
            logging.debug(message)
        elif type == 'info':
            logging.info(message)
        elif type == 'warning':
            logging.warning(message)
        elif type == 'error':
            logging.error(message)

        self.send_az_app_event(type, message)

        if notify_message:
            self.notifcation_api.send_browser_notification(message=message)

    def send_az_app_event(self, type, message):
        if type not in VALID_MESSAGE_TYPES:
            raise ValidationException(
                f'{type} is not in {VALID_MESSAGE_TYPES}.')

        body = {
            "hostname": node(),
            "script_path": sys.argv[0],
            "arguments": sys.argv[1:],
            "type": type,
            "message": message,
            "rvrbase_version": version
        }

        body_json = json.dumps(body)

        try:
            self.azure_logger_api.post_data(
                body_json, LOG_TYPE_APPLICATION_EVENT, self.workspace_id, self.workspace_prim_key)

        except Exception as error:
            message = 'Failed to log message to azure on {node}. With error {error}.'.format(
                node=node(), error=error)
            logging.error(message)
            self.notifcation_api.send_browser_notification(message=message)

    def send_az_metric(self, log_type, metric_name, value):
        body = {
            "hostname": node(),
            "script_path": sys.argv[0],
            "metric_name": metric_name,
            "value": value,
            "rvrbase_version": version
        }

        body_json = json.dumps(body)
        self.azure_logger_api.post_data(
            body_json, log_type, self.workspace_id, self.workspace_prim_key)

    def q1(self, key):
        return self.config_api.q1(key=key)
